<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Synergy controller
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Synergy_Controller extends Authenticated_Controller
{
	const MODE_VIEW = 1;
	const MODE_EDIT = 2;

	protected function _initHead() 
	{
		$this->template->content->authorized_for_edit = nacoma::allowed();
		$this->template->js_header = $this->add_view('js_header');
		$this->template->js_strings = "var synergy_data = " . Synergy_Model::getRulesJson() . "\n; var actions = " . Synergy_Model::getActionsJson() . ";\n";
		$this->template->js_strings .= "var nagios = ".Synergy_Model::getNgiosObjectsJson().";\n";
		$this->template->css_header = '<link type="text/css" rel="stylesheet" href="' . url::base() . 'application/views/themes/default/synergy/css/synergy.css" />';
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.json2.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.jquery.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.liquidmetal.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.flexselect.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.tipsy.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.colorbox.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.validate.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.address.js');
		$this->xtra_js[] = $this->add_path('/synergy/js/synergy.js');
	}

	public function view() 
	{
		$this->template->content = $this->add_view('synergy/index');
		$this->template->content->mode = self::MODE_VIEW;
		$this->_initHead();
		$this->template->title = _('Monitoring » Business Services » View');
		$this->template->disable_refresh = true;
	}

	public function edit()
	{
		$this->template->content = $this->add_view('synergy/index');
		$this->template->content->mode = self::MODE_EDIT;
		$this->_initHead();
		$this->template->title = _('Monitoring » Business Services » Edit');
		$this->template->disable_refresh = true;
	}

	// Checks if the synergy service is running.
	public function service_check()
	{
		$this->auto_render = false;
		exec('sh /opt/synergy/bin/is_service_running.sh', $output, $exit);
		return print($exit);
	}

	// Attempts to start synergy service if it is not running.
	public function service_start()
	{
		$this->auto_render = false;
		exec('service synergy start');
		// Give the service some time to start before we check if it was started or not.
		sleep(5);
		exec('sh /opt/synergy/bin/is_service_running.sh', $output, $exit);
		return print($exit);
	}

	public function update() 
	{
		$this->auto_render = false;
		$result = array(
			"status" => 200,
			"data" => array()
		);
		try {
			if (!isset($_POST["last_updated"])) {
				throw new Exception("Illegal last_updated argument");
			}
			$client_last_updated = intval($_POST["last_updated"]);
			if ($client_last_updated === 0) {
				throw new Exception("Illegal last_updated argument");
			}
			$last_updated = Synergy_Model::getLastIteration();
			if ($last_updated > $client_last_updated) {
				$result["data"]["updated"] = $last_updated;
				$result["data"]["tree"] = Synergy_Model::getRulesJson($this->user);
			} else {
				$result["status"] = 304;
			}
		} catch (Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return print(json_encode($result));
	}

	public function rule_check()
	{
		$this->auto_render = false;
		$result = array("status" => 200, "data" => false);
		try {
			if (!isset($_GET["rule"])) {
				throw new Exception("You have pass a rule argument");
			}
			$rule = (string) $_GET["rule"];
			if (strlen($rule) === 0) {
				throw new Exception("Illegal rule argument");
			}
			$result["data"] = Synergy_Model::isTopLevelExist($rule);

		} catch(Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return print(json_encode($result));
	}

	public function check()
	{
		$this->auto_render = false;
		$result = array("status" => 200, "data" => false);
		try {
			if (!isset($_GET["service"])) {
				throw new Exception("You have pass a service argument");
			}
			$service = (string) $_GET["service"];
			if (strlen($service) === 0) {
				throw new Exception("Illegal service argument");
			}
			list($host, $desc) = explode("&", $service);
			$live = SynergyLivestatus::getInstance();
			$result["data"] = sizeof($live->query(LiveQuery::select("services")
					->columns("host_name")
					->where("host_name = ?", $host)
					->where("description = ?", $desc))) !== 0;

		} catch(Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return print(json_encode($result));
	}

	public function sync() {
		if (nacoma::allowed() === false) {
			header("HTTP/1.0 401 Unauthorized");
			exit;
		}
		$output = "";
		$this->auto_render = false;
		try {
			$nacomaapi = new NacomaApi_Model();

			$uuid = uniqid();
			if (isset($_POST["update_list"])) {
				$update_list = json_decode($_POST["update_list"], true);
				foreach ($update_list as $rule_name => $rule_json) {
					Synergy_Model::saveRule($rule_json, $rule_name, $uuid);
				}
			}

			if (isset($_POST["del_list"])) {
				Synergy_Model::deleteRulesList($_POST["del_list"], $uuid);
			}
			Synergy_Model::commitActions($uuid);

			if (isset($_POST["add"]) or isset($_POST["del"])) {
				if (isset($_POST["add"])) {
					$add = json_decode($_POST["add"], true);
					$nacomaapi->create_services($add, $output);
				}

				if (isset($_POST["del"])) {
					$del = json_decode($_POST["del"], true);
					$nacomaapi->delete_services($del, $output);
				}
				$nacomaapi->saveconfig($output);
			}
		} catch (Exception $e) {
			return print(json_encode(array("result" => false, "msg" => $e->getMessage())));
		}

		return print(json_encode(array("result" => true, "timestamp" => time())));
	}

}
