<?php defined('SYSPATH') OR die('No direct access allowed.');

class LiveQuery {
	static protected $_ALLOWED_SELECT = array(
		"hosts",
		"services",
		"hostgroups",
		"servicegroups",
		"contactgroups",
		"servicesbygroup",
		"servicesbyhostgroup",
		"hostsbygroup",
		"contacts",
		"commands",
		"timeperiods",
		"downtimes",
		"comments",
		"log",
		"status",
		"columns"
	);
	protected $_select = NULL;
	protected $_columns = array();
	protected $_filter = array();
	protected $_auth_user = NULL;

	static public function select($type) {
		if(in_array($type, self::$_ALLOWED_SELECT)) {
			return new self($type);
		}
		return FALSE;
	}

	public function compile() {
		$res = "GET {$this->_select}\n";

		if (!empty($this->_columns)) {
			$res .= "Columns: ".implode(" ", $this->_columns)."\n";
		}

		if (!empty($this->_filter)) {
			foreach ($this->_filter as $filter) {
				$res .= "Filter: $filter\n";
			}
		}

		if (NULL !== $this->_auth_user) {
			$res .= "AuthUser: {$this->_auth_user}\n";
		}
		return $res;
	}

	public function columns($cols) {
		$this->_columns = is_array($cols) ? $cols : array($cols);
		return $this;
	}

	public function where() {
		if(func_num_args() < 0) return;
		$args = func_get_args();
		$args[0] = str_replace('?', '%s', $args[0]);
		$this->_filter[] = call_user_func_array("sprintf", $args);
		return $this;
	}

	public function user($username) {
		if (!is_string($username)) {
			throw new Exception("Wrong livestatus username");
		}

		if (strlen($username) === 0) {
			throw new Exception("Wrong livestatus username");
		}

		$this->_auth_user = $username;
		return $this;
	}

	private function __construct($type) {
		$this->_select = $type;
	}
}