<?php defined('SYSPATH') OR die('No direct access allowed.');

class Synergy_Model extends Model
{
	protected static $_synergy_path = "/opt/synergy/";
	protected static $__last_iteration = NULL;

	// Returns the user-defined BSMs
	public static function getRuleList() {
		$result = array();
		$rules_dir = self::$_synergy_path . "etc/bps";
		if (!is_dir($rules_dir)) throw new Exception("$rules_dir is not directory.");
		if (FALSE === ($dh = opendir($rules_dir))) throw new Exception("can not open dir $rules_dir.");
		while(FALSE !== ($file = readdir($dh))) {
			if(($file[0] === ".") || (array_pop(explode(".", $file)) !== "lua")) continue;
			$result[] = substr($file, 0, -4);
		}
		natcasesort($result);
		return array_reverse($result);
	}

	// Checks if user is permitted to see the service
	private static function __is_permited_by_services($data, Nagios_auth_Model $auth) {
		if (property_exists($auth, "view_services_root") ? $auth->view_services_root : false) {
			return true;
		}

		// Service can be seen by all users
		if ($data["result"]["service_contacts"] === "*") {
			return true;
		}

		// Users is contact for service
		if (in_array($auth->user, $data["result"]["service_contacts"])) {
			return true;
		}

		return false;
	}

	// Checks if user is permitted to see the host
	private static function __is_permited_by_hosts($data, Nagios_auth_Model $auth) {
		if (property_exists($auth, "view_hosts_root") ? $auth->view_hosts_root : false) {
			return true;
		}

		// Host can be seen by all users
		if ($data["result"]["host_contacts"] === "*") {
			return true;
		}

		// User is contact for host
		if (in_array($auth->user, $data["result"]["host_contacts"])) {
			return true;
		}
		return false;
	}

	// Fetches cached results of BSM and returns if user is permitted to see the result
	private static function __is_permited($rule, Nagios_auth_Model $auth) {
		$li = self::getLastIteration();
		if (!file_exists(self::$_synergy_path . "var/state/$li/top/$rule.json.cache")) {
			return FALSE;
		}
		$top = file_get_contents(self::$_synergy_path . "var/state/$li/top/$rule.json.cache");
		if (strlen($top) === 0) {
			return FALSE;
		}
		$data = json_decode($top, true);
		if (!is_array($data)) {
			return FALSE;
		}
		return self::__is_permited_by_services($data, $auth) and self::__is_permited_by_hosts($data, $auth);
	}

	// Takes a list of rules as argument and returns data if user is permitted to see them
	private static function __is_permited_list($list) {
		$auth = new Nagios_auth_Model();
		foreach($list as $rule) {
			if (!self::__is_permited($rule, $auth)) {
				error_log("Synergy: Try to delete unpermited rule ".$rule);
				return false;
			}
		}
		return true;
	}

	// Pretty printing JSON
	public static function _block_format(array $block, $ident = 1) {
		$result = "{\n";
		$i = str_repeat("\t", $ident);
		$c = 0;
		$size = sizeof($block);
		foreach($block as $key => $value) {
			$sep = ++$c == $size ? "" : ",";
			if(is_array($value)) {
				$add = "";
				if(!is_numeric($key)) {
					$add = $key . " = \n" . str_repeat("\t", $ident);
				}
				$result .= sprintf("$i$add%s$sep\n", self::_block_format($value, $ident + 1));
				continue;
			}
			if($key === "intent") {
				$result .= sprintf("$i%s = %s$sep\n", $key, $value);
				continue;
			}
			$result .= sprintf("$i%s = \"%s\"$sep\n", $key, $value);
		}
		return $result . str_repeat("\t", $ident - 1). "}";
	}

	// Saves rule to /opt/synergy/var/queue/$id
	public static function saveRule(array $rule, $rule_name, $sid) {
		$dir_path = self::$_synergy_path . "var/queue/$sid";
		if (!is_dir($dir_path)) {
			mkdir($dir_path, 0777, true);
		}
		if (file_exists(self::$_synergy_path . "rules/".$rule_name.".lua")) {
			if (!self::__is_permited($rule_name, new Nagios_auth_Model())) {
				return FALSE;
			}
		}
		// Pretty prints JSON and puts content into queue
		return @ file_put_contents("$dir_path/$rule_name.lua", "script = " . self::_block_format($rule));
	}

	// Move saved rules to spool directory
	public static function commitActions($sid)
	{
		$dir_path = self::$_synergy_path . "var/queue/$sid";
		if (!is_dir($dir_path)) {
			return FALSE;
		}
		return system("mv -f $dir_path ".self::$_synergy_path."var/spool/");
	}

	// Deletes cached results for given rule
	public static function removeCache($rule_name) {
		$li = self::getLastIteration();
		$file_full = self::$_synergy_path . "var/state/$li/full/$rule_name.json.cache";
		$file_top = self::$_synergy_path . "var/state/$li/top/$rule_name.json.cache";
		if (!file_exists($file_full)) {
			return false;
		}
		@ unlink($file_full);
		if (!file_exists($file_top)) {
			return false;
		}
		@ unlink($file_top);
		return true;
	}

	// Deletes saved rule
	public static function deleteRule($rule_name) {
		$rule_path = self::$_synergy_path . "etc/bps/$rule_name.lua";
		if (!file_exists($rule_path)) return false;
		self::removeCache($rule_name);
		@ unlink($rule_path);
		return true;
	}

	// Deletes list of saved rules
	public static function deleteRulesList($list, $sid) {
		foreach ($list as $rule) {
			if (file_exists(self::$_synergy_path . "etc/bps/".$rule.".lua")) {
				if (!self::__is_permited($rule, new Nagios_auth_Model())) {
					return FALSE;
				}
				self::removeCache($rule);
			}
		}
		$dir_path = self::$_synergy_path . "var/queue/$sid";
		if (!is_dir($dir_path)) {
			mkdir($dir_path, 0777, true);
		}
		return @ file_put_contents("$dir_path/delete.list", json_encode(array_values($list)));
	}

	// Fetches cached results as JSON and returns to browser
	public static function getRulesJson() {
		$li = self::getLastIteration();
		$auth = new Nagios_auth_Model();
		$json = array();
		$rules = self::getRuleList();
		$res = "{\"items\":{"; $c = 0;
		foreach($rules as $rule) {
			if (self::__is_permited($rule, $auth)) {
				if ($c != 0) {
					$res .= ",";
				} else {
					$c = 1;
				}
				$res .= "\"$rule\":".file_get_contents(self::$_synergy_path."var/state/$li/full/$rule.json.cache");
			}
		}
		return $res."}}";
	}

	// Fetches top-level of the cached results for all rules
	public static function getTopsList() {
		$li = self::getLastIteration();
		$auth = new Nagios_auth_Model();
		$list = self::getRuleList();
		$result = array();
		foreach($list as $rule) {
			if (!file_exists(self::$_synergy_path . "var/state/$li/top/$rule.json.cache")) {
				continue;
			}
			$top = file_get_contents(self::$_synergy_path . "var/state/$li/top/$rule.json.cache");
			if (!strlen($top)) {
				continue;
			}
			if (self::__is_permited($rule, $auth)) {
				$result[$rule] = json_decode($top, true);
			}
		}
		return $result;
	}

	// Fetches top-level results for given rule
	public static function getTop($name) {
		$li = self::getLastIteration();
		$auth = new Nagios_auth_Model();
		$result = NULL;
		if (!file_exists(self::$_synergy_path . "var/state/$li/top/$name.json.cache")) {
			return FALSE;
		}
		$top = file_get_contents(self::$_synergy_path . "var/state/$li/top/$name.json.cache");
		if (!strlen($top)) {
			return FALSE;
		}
		if (!self::__is_permited($name, $auth)) {
			return FALSE;
		}
		return $result;
	}

	// Checks if given rule exists
	public static function isTopLevelExist($rule) {
		return file_exists(self::$_synergy_path."etc/bps/$rule.lua");
	}

	// Fetches the different models that can be used to create rules. i.e best_state etc.
	public static function getActionsJson() {
		$actions_path = self::$_synergy_path . "etc/rules/repo";
		if(!is_dir($actions_path)) return FALSE;
		if(!($dir = opendir($actions_path))) return FALSE;
		$result = array();
		while(($file = readdir($dir)) !== FALSE) {
			if(strstr($file, ".lua") && $file[0] !== ".") {
				$result[] = substr($file, 0, -4);
			}
		}
		$custom_actions_path = self::$_synergy_path . "etc/rules/custom/repo";
		if(!is_dir($custom_actions_path)) return FALSE;
		if(!($dir = opendir($custom_actions_path))) return FALSE;
		while(($file = readdir($dir)) !== FALSE) {
			if(strstr($file, ".lua") && $file[0] !== ".") {
				$result[] = substr($file, 0, -4);
			}
		}
		$actions = addslashes(json_encode(array_unique($result)));
		$json = array();
		$status = 0;
		exec("echo $actions | " . self::$_synergy_path . "bin/actions --all", $json, $status);
		if($status !== 0)
			return FALSE;
		return implode("",$json);
	}

	// Fetches the timestamp of last check iteration
	public static function getLastIteration() {
		if (self::$__last_iteration === NULL) {
			$res = @ file_get_contents(self::$_synergy_path . "var/state/iteration");
			if (FALSE === $res) {
				return $res;
			}
			if (0 === ($res = intval($res))) {
				return FALSE;
			}
			self::$__last_iteration = $res;
		}
		return self::$__last_iteration;
	}

	public static function getNgiosObjectsJson() {
		$auth = new Nagios_auth_Model();

		$live = SynergyLivestatus::getInstance();
		$result = array(
			"hosts" => array(),
			"services" => array(),
			"hostgroups" => array(),
			"servicegroups" => array()
		);
		$host_query = LiveQuery::select("hosts")->columns("name");
		if (!$auth->view_hosts_root) {
			$host_query->user($auth->user);
		}
		$hosts = $live->query($host_query);
		$service_query = LiveQuery::select("services")->columns(array("host_name", "description"));
		if (!$auth->view_services_root) {
			$service_query->user($auth->user);
		}
		$services = $live->query($service_query);
		$hostgroup_query = LiveQuery::select("hostgroups")->columns("name");
		if (!$auth->view_hosts_root) {
			$hostgroup_query->user($auth->user);
		}
		$hostgroups = $live->query($hostgroup_query);
		$servicegroup_query = LiveQuery::select("servicegroups")->columns("name");
		if (!$auth->view_services_root) {
			$servicegroup_query->user($auth->user);
		}
		$servicegroups = $live->query($servicegroup_query);
		$live->close();
		foreach ($hosts as $host) {
			$result["hosts"][$host[0]] = $host[0];
		}
		foreach ($services as $host_desc) {
			list($host, $desc) = $host_desc;
			if (!isset($result["services"][$host])) {
				$result["services"][$host] = array();
			}
			array_push($result["services"][$host], $desc);
		}
		foreach ($hostgroups as $hostgroup) {
			$result["hostgroups"][$hostgroup[0]] = $hostgroup[0];
		}
		foreach ($servicegroups as $servicegroup) {
			$result["servicegroups"][$servicegroup[0]] = $servicegroup[0];
		}
		return json_encode($result);
	}
}
