local _G = _G
module(...)
at_least = {
	name = "At least",
	tag = "synergy_group",
	order = 4,
	description = "Result depends on OK thresholds",
	rule = function(items, count_ok, count_wr, unit)
		local c_ok = 0
		local n = 0
		local res_st = 2
		local res_msg 
		local tmpl_msg = "Got %s OK(s) - meeting %s range"
		local state_map = { "OK", "WARNING", "CRITICAL" }
		for i,value in _G.ipairs(items) do
			if value.status == 0 then c_ok = c_ok + 1 end
			n = n + 1
		end
		if n == 0 then return {{ status = 3, msg = "No state, default state UNKNOWN" }} end 
		if unit == "%" then
			local P_ok =_G.math.ceil(100*c_ok/n)
			if P_ok >= count_ok then res_st = 0
			elseif P_ok >= count_wr then res_st = 1
			end
			res_msg = _G.string.format(tmpl_msg, P_ok.."%", state_map[res_st+1])
		else
			if n < count_ok then return {{ status = 2, msg = "Not enough subelements to be OK, defined - state CRITICAL" }} end
			if c_ok >= count_ok then res_st = 0
			elseif c_ok >= count_wr then res_st = 1
			end
			res_msg = _G.string.format(tmpl_msg, c_ok, state_map[res_st+1])
		end
		return {{ status = res_st, msg = res_msg }}
	end,
	params = {
		{
			validation = {
				required = true,
				number = true
			},
			name = "count_ok",
			label = "OK threshold",
			type = "number",
			hint = "Count of OKs to remain OK"
		},
		{
			validation = {
				required = true,
				number = true
			},
			name = "count_wr",
			label = "WARNING threshold",
			type = "number",
			hint = "Count of OKs to remain WARNING"
		},
		{
			validation = {
				required = true
			},
			name = "unit",
			label = "Unit",
			type = "enum",
			hint = "Unit for the above value (percentage or actual number)",
			values = {"%", "num"}
		}
	}
}
