local _G = _G
module(...)
host = {
    name = "Host",
    tag = "nagios",
    order = 10,
    description = "Monitor host state",
    hint = "get status of host",
    links = {
        { href = "/extinfo/details/host/{{nagios_host}}", title = "host details" },
        { href = "/configuration/configure/host/{{nagios_host}}", title = "host configuration" },
        { href = "/status/service/{{nagios_host}}", title = "host services" }
    },
    rule = function(items, name, type)
        local h = _G.host(name)
        if h == nil then
			return {{
				status = 3, 
				msg = "Host not found", 
				service_contacts = "*",
				service_contact_groups = "*",
				host_contacts = "*",
				host_contact_groups = "*"
			}}
		end

		local state, state_change
		if (type == "all") or (h.state_type == 1) then
			state = h.state
			state_change = h.last_state_change
		else
			state = h.last_hard_state
			state_change = h.last_hard_state_change
		end

		--there are not OK, WARNING and CRITICAL states in host, just OK and Down,
		--so convert WARNING to CRITICAL
		if state == 1 then
			state = 2
		end
        return {{
            status = state, 
            msg = h.plugin_output, 
            last_check = h.last_check, 
            last_state_change = state_change,
            host_contacts = h.contacts,
            host_contact_groups = h.contact_groups,
            service_contacts = "*",
            service_contact_groups = "*"
        }}
    end,
    params = {
        {
            validation = {
                required = true,
                unique_in_scope = true
            },
            name = "nagios_host",
            label = "Host",
            type = "nagios_host",
            hint = "Name of host"
        }
    }
}
