local _G = _G
module(...)
service = {
    name = "Service",
    tag = "nagios",
    order = 7,
    description = "Monitor service state",
    hint = "Nagios service",
    links = {
        { href = "/extinfo/details/service/{{nagios_service.name}}?service={{nagios_service.description}}", title = "service details" },
        { href = "/configuration/configure/service/{{nagios_service.name}}?service={{nagios_service.description}}", title = "service configuration" }
    },
    rule = function(items, name, type)
        local sep = _G.string.find(name, '&')
        local h = _G.string.sub(name, 0, sep - 1)
        local d = _G.string.sub(name, sep + 1, _G.string.len(name))
		local host, srv
		local state, state_change
		host = _G.host(h)
		if host ~= nil then
			srv = host:service(d)
		end
		if host == nil or srv == nil then
			return {{
				status = 3, 
				msg = "Object not found", 
				service_contacts = "*",
				service_contact_groups = "*",
				host_contacts = "*",
				host_contact_groups = "*"
			}}
		end
        if (type == "all") or (srv.state_type == 1) then
            state = srv.state
			state_change = srv.last_state_change
        else
            state = srv.last_hard_state
			state_change = srv.last_hard_state_change
        end
        return {{
            status = state, 
            msg = srv.plugin_output, 
            last_check = srv.last_check,
            last_state_change = state_change,
            service_contacts = srv.contacts,
            service_contact_groups = srv.contact_groups,
            host_contacts = "*",
            host_contact_groups = "*"
        }}
    end,
    params = {
        {
            validation = {
                required = true,
                unique_in_scope = true
            },
            name = "nagios_service",
            label = "Service",
            type = "nagios_service",
            hint = "Host name and service description"
        }
    }
}
